
from pyglet import clock
from math import sqrt
from math import fabs
from math import ceil
from math import floor
from math import e
from math import pi
from math import cos
from math import sin
import random
from common import *

class Blob(Sprite):
	
	def __init__(self, leaf_list, min_radius, parent_window):
		self.parent_window = parent_window
		self.leaves = leaf_list
		self.active = True
		self.timer_active = False
		self.x = 0
		self.y = 0
		self.radius = 0
		self.min_radius = min_radius
		self.limit_size	= 15	#smaller number = limiting starts at lower size
		self.update_shape()
		for leaf in leaf_list:
			leaf.attach_blob(self, False)
		#print "Blob created at ",self.x,self.y,"with radius",self.radius
		
		self.font = font.load('Arial', 10)
		self.info_text = font.Text(self.font, y=self.y-self.radius, x=self.x+self.radius)
	
	def update(self):
		if (self.active):
			self.update_shape()
			self.absorption_check()
			
			for leaf in self.leaves:
				# Has leaf left blob area?
				if (leaf.distance(self) > self.radius):
					leaf.unattach_blob(True)
			
			# Timer Control
			if (not self.timer_active and len(self.leaves) > 3):
				clock.schedule_once(self.on_time_up, 10)
				self.timer_active = True
			elif (len(self.leaves) < 4):
				clock.unschedule(self.on_time_up)
				self.timer_active = False
	
	def draw(self):
		verts = int( self.radius )
		v2f = []
		for i in range(int(verts)):
			v2f.append( self.x+5 + cos( (i/float(verts)) * 2 * pi ) * self.radius )
			v2f.append( self.y+10 + sin( (i/float(verts)) * 2 * pi ) * self.radius )
		graphics.draw(verts, gl.GL_POINTS, ('v2f', v2f))
		
		self.info_text.x = self.x + self.radius
		self.info_text.y = self.y - self.radius
		self.info_text.text = ("%d") % (len(self.leaves)) 
		self.info_text.draw()
	
	def attach_leaf(self, leaf):
		self.leaves.append(leaf)
	
	def unattach_leaf(self, leaf):
		self.leaves.remove(leaf)
		# Does the blob now contain less than 2 leaves?
		if (len(self.leaves) < 2):
			for leaf in self.leaves:
				leaf.unattach_blob(False)
			self.parent_window.kill_blob(self)
	
	def update_shape(self):
		if (len(self.leaves)<2):
			return
		temp_x = 0
		temp_y = 0
		for leaf in self.leaves:
			temp_x += leaf.x
			temp_y += leaf.y
		self.x = temp_x / len(self.leaves)
		self.y = temp_y / len(self.leaves)
		if ( len(self.leaves) > self.limit_size):
			limiter = self.limit_size / len(self.leaves)
		self.radius = self.min_radius + len(self.leaves)
	
	def absorption_check(self):
		for blob in self.parent_window.blobs:
			if (blob is not self and blob.distance(self) < self.radius):
				blob.deactivate()
				clock.unschedule(blob.on_time_up)
				for leaf in blob.leaves:
					leaf.unattach_blob(False)
				self.parent_window.kill_blob(blob)
	
	def deactivate(self):
		self.active = False
	
	def on_time_up(self, dt):
		self.parent_window.player.add_points( 4 * 2**(len(self.leaves)-4) )
		for leaf in self.leaves:
			self.parent_window.kill_leaf(leaf)
		self.parent_window.kill_blob(self)
	